/// <reference types="sequelize" />
import * as Sequelize from 'sequelize';
import { Spec } from 'pretur.spec';
import { Resolver, UnitializedResolver } from './resolver';
import { Synchronizer, UnitializedSynchronizer } from './synchronizer';
import { UninitializedSequelizeModel } from './sequelizeModel';
import { Pool } from './pool';
export declare type FieldWhereClause = string | number | Sequelize.WhereOptions | Sequelize.WhereOptions | Sequelize.col | Sequelize.and | Sequelize.or | Sequelize.WhereGeometryOptions | (string | number)[] | Object;
export interface AliasModelMap {
    [alias: string]: string;
}
export interface AliasKeyMap {
    [alias: string]: string;
}
export interface AliasRequiredMap {
    [alias: string]: boolean;
}
export interface FieldWhereBuilders {
    [field: string]: (filter: any) => FieldWhereClause;
}
export interface ModelDescriptor<T> {
    spec: Spec<T>;
    name: string;
    primaryKey?: string;
    sequelizeModel?: Sequelize.Model<any, any>;
    resolver?: Resolver<any>;
    synchronizer?: Synchronizer<any>;
    aliasModelMap: AliasModelMap;
    aliasKeyMap: AliasKeyMap;
    aliasRequiredMap: AliasRequiredMap;
    allowedAttributes: string[];
    mutableAttributes: string[];
    defaultOrder: [string, 'ASC' | 'DESC'];
    fieldWhereBuilders?: FieldWhereBuilders;
    sanitizeAttributes(attributes?: string[] | string): string[];
    initialize(pool: Pool): void;
}
export interface BuildModelDescriptorOptions {
    sequelizeModel?: UninitializedSequelizeModel<any, any>;
    resolver?: UnitializedResolver<any>;
    synchronizer?: UnitializedSynchronizer<any>;
    defaultOrder?: [string, 'ASC' | 'DESC'];
    fieldWhereBuilders?: FieldWhereBuilders;
    allowedAttributes?: string[];
    allowedInsertAttributes?: string[];
    allowedUpdateAttributes?: string[];
}
export declare function buildModelDescriptor<T>(spec: Spec<T>, options?: BuildModelDescriptorOptions): ModelDescriptor<T>;
