"use strict";
var Bluebird = require('bluebird');
function createPool() {
    var descriptors = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        descriptors[_i - 0] = arguments[_i];
    }
    var pool = {
        models: descriptors.reduce(function (m, d) {
            m[d.name] = d;
            return m;
        }, {}),
    };
    pool.resolve = function resolve(query) {
        if (!query || !query.model) {
            return Bluebird.reject(new Error('query or query.model was not specified.'));
        }
        if (!pool.models[query.model]) {
            return Bluebird.reject(new Error(query.model + " is not a valid model"));
        }
        return pool.models[query.model].resolver(query);
    };
    pool.sync = function sync(transaction, item, rip) {
        if (!item.model) {
            return Bluebird.reject(new Error('item.model was not specified.'));
        }
        if (!pool.models[item.model]) {
            return Bluebird.reject(new Error(item.model + " is not a valid model"));
        }
        return pool.models[item.model].synchronizer(transaction, item, rip);
    };
    descriptors.forEach(function (d) { return d.initialize(pool); });
    return pool;
}
exports.createPool = createPool;
