/// <reference types="bluebird" />
/// <reference types="sequelize" />
import * as Bluebird from 'bluebird';
import * as Sequelize from 'sequelize';
import { I18nBundle } from 'pretur.i18n';
import { Spec } from 'pretur.spec';
import { Pool } from './pool';
import { SynchronizerItem, SynchronizerInsert, SynchronizerUpdate, SynchronizerRemove } from 'pretur.sync';
export interface ResultItemAppender {
    appendError(error: I18nBundle): void;
    appendWarning(warning: I18nBundle): void;
}
export interface ErrorHandler<T> {
    (data: T, error: any, rip: ResultItemAppender): Bluebird<void>;
}
export interface Insert<T> {
    (transaction: Sequelize.Transaction, item: SynchronizerInsert<T>, rip: ResultItemAppender): Bluebird<void>;
}
export interface Update<T> {
    (transaction: Sequelize.Transaction, item: SynchronizerUpdate<T>, rip: ResultItemAppender): Bluebird<void>;
}
export interface Remove {
    (transaction: Sequelize.Transaction, item: SynchronizerRemove, rip: ResultItemAppender): Bluebird<void>;
}
export interface Synchronizer<T> {
    (transaction: Sequelize.Transaction, item: SynchronizerItem<T>, rip: ResultItemAppender): Bluebird<void>;
}
export interface UnitializedSynchronizer<T> {
    synchronizer: Synchronizer<T>;
    initialize(pool: Pool): void;
}
export interface SynchronizationInterceptor<T> {
    (transaction: Sequelize.Transaction, item: T, rip: ResultItemAppender, pool: Pool): Bluebird<boolean>;
}
export interface BuildSynchronizerOptions<T> {
    insertErrorHandler?: ErrorHandler<SynchronizerInsert<T>>;
    updateErrorHandler?: ErrorHandler<SynchronizerUpdate<T>>;
    removeErrorHandler?: ErrorHandler<SynchronizerRemove>;
    insertInterceptor?: SynchronizationInterceptor<SynchronizerInsert<T>>;
    updateInterceptor?: SynchronizationInterceptor<SynchronizerUpdate<T>>;
    removeInterceptor?: SynchronizationInterceptor<SynchronizerRemove>;
}
export declare function buildSynchronizer<T>(spec: Spec<T>, options?: BuildSynchronizerOptions<T>): UnitializedSynchronizer<T>;
