"use strict";
var Bluebird = require('bluebird');
var lodash_1 = require('lodash');
function buildSynchronizer(spec, options) {
    var pool = null;
    function synchronizer(transaction, item, rip) {
        if (item.action === 'INSERT') {
            return insert(pool, spec.name, (options && options.insertErrorHandler) || undefined, (options && options.insertInterceptor) || undefined, transaction, item, rip);
        }
        if (item.action === 'UPDATE') {
            return update(pool, spec.name, (options && options.updateErrorHandler) || undefined, (options && options.updateInterceptor) || undefined, transaction, item, rip);
        }
        if (item.action === 'REMOVE') {
            return remove(pool, spec.name, (options && options.removeErrorHandler) || undefined, (options && options.removeInterceptor) || undefined, transaction, item, rip);
        }
        return Bluebird.reject(new Error('provided item is invalid'));
    }
    function initialize(p) {
        pool = p;
    }
    return { synchronizer: synchronizer, initialize: initialize };
}
exports.buildSynchronizer = buildSynchronizer;
var INJECTED_MASTER_RESOLUTION_KEY = '__INJECTED_MASTER_RESOLUTION_KEY';
function insert(pool, model, errorHandler, interceptor, transaction, item, rip) {
    function defaultInsertBehavior() {
        var data = lodash_1.assign({}, item.data);
        var masters = [];
        pool.models[model].spec.relations.master.forEach(function (master) {
            var masterData = data[master.alias];
            if (masterData) {
                var masterModel = pool.models[master.model];
                masters.push(masterModel.synchronizer(transaction, (_a = {
                        action: 'INSERT',
                        data: masterData,
                        itemId: item.itemId,
                        model: masterModel.name
                    },
                    _a[INJECTED_MASTER_RESOLUTION_KEY] = function (id) { return data[master.key] = id; },
                    _a
                ), rip));
                data[master.alias] = null;
            }
            var _a;
        });
        var modelCreationPromise = Bluebird.all(masters).then(function () {
            return pool.models[model].sequelizeModel.create(data, {
                transaction: transaction,
                fields: pool.models[model].allowedAttributes,
            });
        });
        var promise = modelCreationPromise.then(function (instance) {
            if (item[INJECTED_MASTER_RESOLUTION_KEY]) {
                item[INJECTED_MASTER_RESOLUTION_KEY](instance[pool.models[model].primaryKey]);
            }
            var details = [];
            var aliasModelMap = pool.models[model].aliasModelMap;
            var aliasKeyMap = pool.models[model].aliasKeyMap;
            Object.keys(aliasModelMap).forEach(function (alias) {
                var nested = data[alias];
                if (nested) {
                    if (Array.isArray(nested)) {
                        nested.forEach(function (nestedItem) {
                            var nestedInsertData = lodash_1.assign({}, nestedItem, (_a = {}, _a[aliasKeyMap[alias]] = instance[pool.models[model].primaryKey], _a));
                            details.push(pool.models[aliasModelMap[alias]].synchronizer(transaction, {
                                action: 'INSERT',
                                data: nestedInsertData,
                                itemId: item.itemId,
                                model: aliasModelMap[alias],
                            }, rip));
                            var _a;
                        });
                    }
                    else {
                        var nestedInsertData = lodash_1.assign({}, nested, (_a = {}, _a[aliasKeyMap[alias]] = instance[pool.models[model].primaryKey], _a));
                        details.push(pool.models[aliasModelMap[alias]].synchronizer(transaction, {
                            action: 'INSERT',
                            data: nestedInsertData,
                            itemId: item.itemId,
                            model: aliasModelMap[alias],
                        }, rip));
                    }
                }
                var _a;
            });
            return details.length > 0 ? Bluebird.all(details) : undefined;
        });
        if (typeof errorHandler === 'function') {
            return promise.catch(function (error) { return errorHandler(item, error, rip); });
        }
        return promise;
    }
    if (typeof interceptor === 'function') {
        return interceptor(transaction, item, rip, pool).then(function (resume) {
            if (resume) {
                return defaultInsertBehavior();
            }
        });
    }
    return defaultInsertBehavior();
}
function update(pool, model, errorHandler, interceptor, transaction, item, rip) {
    function defaultUpdateBehavior() {
        var primaryKey = pool.models[model].primaryKey;
        var promise = pool.models[model].sequelizeModel.update(item.data, {
            transaction: transaction,
            fields: buildUpdateAttributes(pool.models[model].mutableAttributes, item.attributes),
            validate: true,
            where: (_a = {}, _a[primaryKey] = item.data[primaryKey], _a),
        });
        if (typeof errorHandler === 'function') {
            return promise.catch(function (error) { return errorHandler(item, error, rip); });
        }
        return promise;
        var _a;
    }
    if (typeof interceptor === 'function') {
        return interceptor(transaction, item, rip, pool).then(function (resume) {
            if (resume) {
                return defaultUpdateBehavior();
            }
        });
    }
    return defaultUpdateBehavior();
}
function remove(pool, model, errorHandler, interceptor, transaction, item, rip) {
    function defaultRemoveBehavior() {
        var promise = pool.models[model].sequelizeModel.destroy({
            transaction: transaction,
            where: { id: item.targetId },
        });
        if (typeof errorHandler === 'function') {
            return promise.catch(function (error) { return errorHandler(item, error, rip); });
        }
        return promise;
    }
    if (typeof interceptor === 'function') {
        return interceptor(transaction, item, rip, pool).then(function (resume) {
            if (resume) {
                return defaultRemoveBehavior();
            }
        });
    }
    return defaultRemoveBehavior();
}
function buildUpdateAttributes(allowedAttributes, updateAttributes) {
    if (Array.isArray(allowedAttributes)) {
        return lodash_1.intersection(allowedAttributes, updateAttributes);
    }
    return updateAttributes;
}
